import os
import sys
from pathlib import Path

class Config(object):
    class Forced:
        def __init__(self, config):
            self.config = config
            self.force = config.force

        def __enter__(self):
            self.config.force = True
            return self.config

        def __exit__(self, type, value, traceback):
            self.config.force = self.force

    def __init__(self, quiet=False, project_name=None, force=False, verbose=False):
        from .consts import YAML_VERSION
        from . import odoo_config  # NOQA

        from .init_functions import _get_customs_root
        self.WORKING_DIR = _get_customs_root(Path(os.getcwd()))
        self._host_run_dir = None
        self.project_name = project_name
        self.YAML_VERSION = YAML_VERSION
        self.verbose = verbose
        self.force = force
        self.compose_version = YAML_VERSION
        self.quiet = quiet
        self.restrict = []

    @property
    def project_name(self):
        return self._project_name

    @project_name.setter
    def project_name(self, value):
        self._project_name = value
        if self._project_name:
            os.environ['PROJECT_NAME'] = value
            self.HOST_RUN_DIR = Path(os.environ['HOME']) / '.odoo' / 'run' / value
        else:
            os.environ['PROJECT_NAME'] = ""
        self.setup_files_and_folders()

    @property
    def HOST_RUN_DIR(self):
        return self._host_run_dir

    @HOST_RUN_DIR.setter
    def HOST_RUN_DIR(self, value):
        self._host_run_dir = value
        if value:
            os.environ['HOST_RUN_DIR'] = str(value)
        self.setup_files_and_folders()

    def setup_files_and_folders(self):
        from .init_functions import get_use_docker
        from . import odoo_config  # NOQA
        self.dirs = {}
        self.files = {}
        self.commands = {}

        self.use_docker = get_use_docker(self.files)
        from .init_functions import make_absolute_paths
        make_absolute_paths(self, self.dirs, self.files, self.commands)

        from .program_settings import ProgramSettings
        self.runtime_settings = ProgramSettings(self.files['runtime_settings'])

    def forced(self):
        return Config.Forced(self)

    def __getattribute__(self, name):
        try:
            value = super(Config, self).__getattribute__(name)
            return value
        except AttributeError:
            from .myconfigparser import MyConfigParser  # NOQA
            if 'settings' not in self.files:
                return None
            myconfig = MyConfigParser(self.files['settings'])

            convert = None
            if name.endswith('_as_int'):
                convert = 'asint'
                name = name[:-len('_as_int')]
            elif name.endswith('_as_bool'):
                convert = 'asbool'
                name = name[:-len('_as_bool')]

            for tries in [name, name.lower(), name.upper()]:
                value = ''
                if tries not in myconfig.keys():
                    continue

                value = myconfig.get(tries, "")
                break

            if convert:
                if convert == 'asint':
                    value = int(value or '0')

            if value == "1":
                value = True
            elif value == "0":
                value = False
            return value
        except Exception:
            raise

    def get_odoo_conn(self):
        from .odoo_config import get_postgres_connection_params # NOQA
        from .tools import DBConnection
        host, port, user, password = get_postgres_connection_params()
        conn = DBConnection(
            self.dbname,
            host,
            port,
            user,
            password
        )
        return conn



import json
from pathlib import Path

class ProgramSettings(object):
    def __init__(self, filename):
        self.filename = Path(filename)

    def get(self, name, default_value=None):
        if not self.filename.exists():
            self.filename.write_text("{}")
        data = json.loads(self.filename.read_text() or "{}")
        return data.get(name, default_value)

    def set(self, name, value):
        if isinstance(value, Path):
            value = str(value)
        data = json.loads(self.filename.read_text() or "{}")
        data[name] = value
        self.filename.write_text(json.dumps(data))



from pathlib import Path
from .init_functions import _search_path
VERSIONS = [7.0, 8.0, 9.0, 10.0, 11.0, 12.0, 13.0]
YAML_VERSION = '3.7'

default_dirs = {
    'admin': 'admin',
    'odoo_home': '',
    'proxy_configs_dir': '${run}/proxy',
    'run': '${run}',
    'run/proxy': '${run}/proxy',
    'run/restore': '${run}/restore',
    'images/proxy': 'images/proxy',
    'telegrambot': 'config/telegrambat',
    'venv': "${run}/venv",
    'run_native_config_dir': '${run}/configs',
    'run_native_bin_dir': '${run}/bin',
    'run_native_requirements': '${run}/requirements', # requirement files
    'run_native_out_dir': '${run}/odoo_outdir',
    'odoo_tools': '$odoo_home',
    'odoo_data_dir': "~/.odoo/files",
    'user_conf_dir': "~/.odoo",
    'cicd_delegator': '~/.odoo/cicd_delegator',
    'images': '~/.odoo/images',
}

default_files = {
    'after_reload_script': "/usr/local/bin/after-odoo-reload.sh",
    'after_up_script': "/usr/local/bin/after-odoo-up.sh",
    'settings_auto': "${run}/settings.auto",
    'project_settings': "~/.odoo/settings.${project_name}",
    'project_docker_compose.home': "~/.odoo/docker-compose.yml",
    'project_docker_compose.home.project': "~/.odoo/docker-compose.${project_name}.yml",
    'project_docker_compose.local': "${working_dir}/.odoo/docker-compose.${project_name}.yml",
    'docker_compose': '${run}/docker-compose.yml',
    'docker_compose_bin': _search_path('docker-compose'),
    'debugging_template_withports': 'config/template_withports.yml',
    'debugging_template_onlyloop': 'config/template_onlyloop.yml',
    'debugging_composer': '${run}/debugging.yml',
    'settings': '${run}/settings',
    'odoo_instances': '${run}/odoo_instances',
    'config/default_network': 'config/default_network',
    'config/cicd_network': 'config/cicd_network_for_project.yml',
    'run/odoo_debug.txt': '${run}/debug/odoo_debug.txt',
    'run/snapshot_mappings.txt': '${run}/snapshot_mappings.txt',
    'images/proxy/instance.conf': 'images/proxy/instance.conf',
    'commit': 'odoo.commit',
    'native_bin_install_requirements': "${run_native_bin_dir}/install-requirements",
    'native_bin_restore_dump': "${run_native_bin_dir}/restore-db",
    'native_collected_requirements_from_modules': "${run_native_bin_dir}/customs-requirements.txt",
    'start-dev': '~/.odoo/start-dev',
    'cicd_delegator_registry': '${cicd_delegator}/registry.json',
    'runtime_settings': '~/.odoo/.script_settings',
}

default_commands = {
    'dc': ['${docker_compose_bin}', "-p", "${project_name}", "-f",  "${docker_compose}"],
}
